using System;
using System.Collections.Generic;
using System.Linq;
using Niche.GroupInspector.Domain;
using Niche.Shared;

namespace Niche.GroupInspector.Presentation
{
    /// <summary>
    /// Implementation of IGroupInspectorPresenter, used to provide the 
    /// functionality of our GroupInspector.
    /// </summary>
    public class GroupInspectorPresenter : IGroupInspectorPresenter
    {
        /// <summary>
        /// Gets the model from which we source our data
        /// </summary>
        public IGroupInspectorModel Model
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the view provided for user interaction
        /// </summary>
        public IGroupInspectorView View
        {
            get;
            private set;
        }

        /// <summary>
        /// Gets the ordering configured by the user
        /// </summary>
        public ViewOrder ViewOrder
        {
            get;
            private set;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="GroupInspectorPresenter"/> class.
        /// </summary>
        /// <param name="model">Model to display.</param>
        /// <param name="view">View for the user.</param>
        public GroupInspectorPresenter(IGroupInspectorModel model, IGroupInspectorView view)
        {
            model.RequireNotNull("model");
            view.RequireNotNull("view");

            Model = model;
            View = view;
            View.Presenter = this;

            ViewOrder = ViewOrder.FamilyName;
        }

        /// <summary>
        /// Configure our View for initial display
        /// </summary>
        public void InitializeView()
        {
            View.SetViewStyle(ViewStyle.Tiles);
            View.DisplayError(string.Empty, string.Empty);
            var availableSystems = Model.LoadApplicationSystems();
            View.SetAvailableSystems(availableSystems);
        }

        /// <summary>
        /// Select the passed Application system
        /// </summary>
        /// <param name="system">Application system to select</param>
        public void SelectApplicationSystem(ApplicationSystem system)
        {
            system.RequireNotNull("system");

            mCurrentSystem = system;

            DisplayApplicationSystem(system);
        }

        /// <summary>
        /// Configure our view to display a particular style
        /// </summary>
        /// <param name="viewStyle">Style of view to configure</param>
        public void SetViewStyle(ViewStyle viewStyle)
        {
            View.SetViewStyle(viewStyle);
        }

        /// <summary>
        /// Configure our view to display in a particular order
        /// </summary>
        /// <param name="viewOrder">Specify the order of display</param>
        public void SetOrder(ViewOrder viewOrder)
        {
            ViewOrder = viewOrder;

            if (mCurrentSystem != null)
            {
                DisplayApplicationSystem(mCurrentSystem);
            }
        }

        /// <summary>
        /// Display a specified Application System
        /// </summary>
        /// <param name="system">System to display</param>
        public void DisplayApplicationSystem(ApplicationSystem system)
        {
            system.RequireNotNull("system");

            var errorMessage = string.Empty;
            var errorTooltip = string.Empty;

            int totalGroups = system.Groups.Count();
            int loadedGroups = 0;

            foreach (var g in system.Groups)
            {
                View.DisplayProgress("Loading " + g.DisplayName, loadedGroups, totalGroups);
                try
                {
                    var users = Model.GetUsersForAccessGroup(g);
                    g.Users = OrderUsersBy(users, ViewOrder).ToList();
                }
                catch (Exception e)
                {
                    errorMessage
                        = string.Format(
                            "While displaying groups for {0}, failed to find members of {1} ({2})",
                            system.Name,
                            g.DisplayName,
                            g.Name);
                    errorTooltip = e.Message;
                    break;
                }

                loadedGroups++;
            }

            View.DisplayAccessGroups(system.Groups);

            View.DisplayProgress("Loaded " + system.Name, totalGroups, totalGroups);
            View.DisplayError(errorMessage, errorTooltip);
        }

        /// <summary>
        /// Resequence found users into the desired order
        /// </summary>
        /// <param name="users">Users found</param>
        /// <param name="order">Desired order</param>
        /// <returns>Reordered sequence of users</returns>
        public IEnumerable<User> OrderUsersBy(IEnumerable<User> users, ViewOrder order)
        {
            users.RequireNotNull("users");
            switch (order)
            {
                case ViewOrder.FamilyName:
                    return users.OrderBy(u => u.Surname);

                case ViewOrder.Name:
                    return users.OrderBy(u => u.Name);

                case ViewOrder.UserName:
                    return users.OrderBy(u => u.UserName);

                default:
                    return users;
            }
        }

        /// <summary>
        /// Storage for the currently selected system.
        /// </summary>
        private ApplicationSystem mCurrentSystem;
    }
}